@@ -1,9 +1,7 @@
 /*
  * This file is part of oFono - Open Source Telephony
  *
- * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
- *
- * Contact: Rémi Denis-Courmont <remi.denis-courmont@nokia.com>
+ * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,6 +36,9 @@ typedef struct _GIsiClient GIsiClient;
 struct _GIsiRequest;
 typedef struct _GIsiRequest GIsiRequest;
 
+typedef void (*GIsiVerifyFunc)(GIsiClient *client, bool alive,
+				uint16_t object, void *opaque);
+
 typedef bool (*GIsiResponseFunc)(GIsiClient *client,
 		const void *restrict data, size_t len,
 		uint16_t object, void *opaque);
@@ -46,13 +47,17 @@ typedef void (*GIsiIndicationFunc) (GIsiClient *client,
 		const void *restrict data, size_t len,
 		uint16_t object, void *opaque);
 
-typedef void (*GIsiDebugFunc) (const void *restrict data, size_t len,
-		void *opaque);
-
 GIsiClient *g_isi_client_create(GIsiModem *modem, uint8_t resource);
 
+GIsiRequest *g_isi_verify(GIsiClient *client, GIsiVerifyFunc func,
+				void *opaque);
+
 uint8_t g_isi_client_resource(GIsiClient *client);
 
+void g_isi_version_set(GIsiClient *client, int major, int minor);
+int g_isi_version_major(GIsiClient *client);
+int g_isi_version_minor(GIsiClient *client);
+
 void g_isi_client_set_debug(GIsiClient *client, GIsiDebugFunc func,
 				void *opaque);
 
@@ -63,6 +68,10 @@ int g_isi_client_error(const GIsiClient *client);
 GIsiRequest *g_isi_request_make(GIsiClient *client, const void *data,
 				size_t len, unsigned timeout,
 				GIsiResponseFunc func, void *opaque);
+struct iovec;
+GIsiRequest *g_isi_request_vmake(GIsiClient *client, const struct iovec *iov,
+				size_t iovlen, unsigned timeout,
+				GIsiResponseFunc func, void *opaque);
 
 void g_isi_request_cancel(GIsiRequest *req);
 
