@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -38,11 +38,13 @@
 
 #include "common.h"
 #include "storage.h"
+#include "idmap.h"
 
 #define DATA_CONNECTION_MANAGER_INTERFACE "org.ofono.DataConnectionManager"
 #define DATA_CONTEXT_INTERFACE "org.ofono.PrimaryDataContext"
 
 #define GPRS_FLAG_ATTACHING 0x1
+#define GPRS_FLAG_RECHECK 0x2
 
 #define SETTINGS_STORE "gprs"
 #define SETTINGS_GROUP "Settings"
@@ -66,14 +68,10 @@ struct ofono_gprs {
 	ofono_bool_t roaming_allowed;
 	ofono_bool_t powered;
 	int status;
-	int location;
-	int cellid;
-	int technology;
 	int flags;
-	int next_context_id;
-	unsigned int cid_min;
-	unsigned int cid_max;
-	unsigned char *cid_bitmap;
+	struct idmap *pid_map;
+	unsigned int last_context_id;
+	struct idmap *cid_map;
 	int netreg_status;
 	struct ofono_netreg *netreg;
 	unsigned int netreg_watch;
@@ -108,6 +106,7 @@ struct pri_context {
 	ofono_bool_t active;
 	enum gprs_context_type type;
 	char name[MAX_CONTEXT_NAME_LENGTH + 1];
+	unsigned int id;
 	char *path;
 	char *key;
 	struct context_settings *settings;
@@ -145,44 +144,12 @@ static enum gprs_context_type gprs_context_string_to_type(const char *str)
 
 static unsigned int gprs_cid_alloc(struct ofono_gprs *gprs)
 {
-	unsigned int i, j;
-	unsigned int num_cids = gprs->cid_max - gprs->cid_min + 1;
-	unsigned int offset = num_cids / 8;
-	unsigned int max_bit = num_cids % 8;
-
-	for (i = 0; i < offset; i++) {
-		for (j = 0; j < 8; j++) {
-			if (gprs->cid_bitmap[i] & (1 << j))
-				continue;
-
-			gprs->cid_bitmap[i] |= (1 << j);
-			return i * 8 + j + 1;
-		}
-	}
-
-	for (j = 0; j < max_bit; j++) {
-		if (gprs->cid_bitmap[offset] & (1 << j))
-			continue;
-
-		gprs->cid_bitmap[i] |= (1 << j);
-		return offset * 8 + j + 1;
-	}
-
-	return 0;
+	return idmap_alloc(gprs->cid_map);
 }
 
 static void gprs_cid_release(struct ofono_gprs *gprs, unsigned int id)
 {
-	unsigned int offset = (id - 1) / 8;
-	unsigned int bit = (id - 1) % 8;
-
-	if (id > gprs->cid_max)
-		return;
-
-	if (id < gprs->cid_min)
-		return;
-
-	gprs->cid_bitmap[offset] &= ~(1 << bit);
+	idmap_put(gprs->cid_map, id);
 }
 
 static struct pri_context *gprs_context_by_path(struct ofono_gprs *gprs,
@@ -447,7 +414,7 @@ static void pri_activate_callback(const struct ofono_error *error,
 	DBG("%p %s", ctx, interface);
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Activating context failed with error: %s",
+		DBG("Activating context failed with error: %s",
 				telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&gc->pending,
 					__ofono_error_failed(gc->pending));
@@ -484,7 +451,7 @@ static void pri_deactivate_callback(const struct ofono_error *error, void *data)
 	dbus_bool_t value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Deactivating context failed with error: %s",
+		DBG("Deactivating context failed with error: %s",
 				telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&gc->pending,
 					__ofono_error_failed(gc->pending));
@@ -683,6 +650,11 @@ static DBusMessage *pri_set_property(DBusConnection *conn,
 	if (g_str_equal(property, "Active")) {
 		struct ofono_gprs_context *gc = ctx->gprs->context_driver;
 
+		if (gc == NULL || gc->driver->activate_primary == NULL ||
+				gc->driver->deactivate_primary == NULL ||
+				ctx->gprs->cid_map == NULL)
+			return __ofono_error_not_implemented(msg);
+
 		if (gc->pending)
 			return __ofono_error_busy(msg);
 
@@ -700,17 +672,14 @@ static DBusMessage *pri_set_property(DBusConnection *conn,
 		if (ctx->gprs->flags & GPRS_FLAG_ATTACHING)
 			return __ofono_error_attach_in_progress(msg);
 
-		if (gc == NULL || gc->driver->activate_primary == NULL ||
-				gc->driver->deactivate_primary == NULL)
-			return __ofono_error_not_implemented(msg);
-
 		if (value) {
 			ctx->context.cid = gprs_cid_alloc(ctx->gprs);
 
 			if (ctx->context.cid == 0)
 				return __ofono_error_failed(msg);
 
-			if (ctx->context.cid != 1) {
+			if (ctx->context.cid !=
+					idmap_get_min(ctx->gprs->cid_map)) {
 				ofono_error("Multiple active contexts are"
 						" not yet supported");
 
@@ -823,17 +792,17 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	char path[256];
-	unsigned int id = ctx->gprs->next_context_id;
 	const char *basepath;
 
 	basepath = __ofono_atom_get_path(ctx->gprs->atom);
 
-	snprintf(path, sizeof(path), "%s/primarycontext%u", basepath, id);
+	snprintf(path, sizeof(path), "%s/primarycontext%u", basepath, ctx->id);
 
 	if (!g_dbus_register_interface(conn, path, DATA_CONTEXT_INTERFACE,
 					context_methods, context_signals,
 					NULL, ctx, pri_context_destroy)) {
 		ofono_error("Could not register PrimaryContext %s", path);
+		idmap_put(ctx->gprs->pid_map, ctx->id);
 		pri_context_destroy(ctx);
 
 		return FALSE;
@@ -842,8 +811,6 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 	ctx->path = g_strdup(path);
 	ctx->key = ctx->path + strlen(basepath) + 1;
 
-	ctx->gprs->next_context_id += 1;
-
 	return TRUE;
 }
 
@@ -853,6 +820,7 @@ static gboolean context_dbus_unregister(struct pri_context *ctx)
 	char path[256];
 
 	strcpy(path, ctx->path);
+	idmap_put(ctx->gprs->pid_map, ctx->id);
 
 	return g_dbus_unregister_interface(conn, path,
 						DATA_CONTEXT_INTERFACE);
@@ -876,79 +844,6 @@ static char **gprs_contexts_path_list(GSList *context_list)
 	return objlist;
 }
 
-static void set_registration_status(struct ofono_gprs *gprs, int status)
-{
-	const char *str_status = registration_status_to_string(status);
-	const char *path = __ofono_atom_get_path(gprs->atom);
-	DBusConnection *conn = ofono_dbus_get_connection();
-
-	if (gprs->status == status)
-		return;
-
-	gprs->status = status;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"Status", DBUS_TYPE_STRING,
-					&str_status);
-}
-
-static void set_registration_location(struct ofono_gprs *gprs,
-					int lac)
-{
-	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = __ofono_atom_get_path(gprs->atom);
-	dbus_uint16_t dbus_lac = lac;
-
-	if (lac > 0xffff)
-		return;
-
-	gprs->location = lac;
-
-	if (gprs->location == -1)
-		return;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"LocationAreaCode",
-					DBUS_TYPE_UINT16, &dbus_lac);
-}
-
-static void set_registration_cellid(struct ofono_gprs *gprs, int ci)
-{
-	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = __ofono_atom_get_path(gprs->atom);
-	dbus_uint32_t dbus_ci = ci;
-
-	gprs->cellid = ci;
-
-	if (gprs->cellid == -1)
-		return;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"CellId", DBUS_TYPE_UINT32,
-					&dbus_ci);
-}
-
-static void set_registration_technology(struct ofono_gprs *gprs,
-					int tech)
-{
-	const char *tech_str = registration_tech_to_string(tech);
-	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = __ofono_atom_get_path(gprs->atom);
-
-	gprs->technology = tech;
-
-	if (gprs->technology == -1)
-		return;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"Technology", DBUS_TYPE_STRING,
-					&tech_str);
-}
-
 static void gprs_attached_update(struct ofono_gprs *gprs)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
@@ -997,33 +892,42 @@ static void gprs_attached_update(struct ofono_gprs *gprs)
 }
 
 static void registration_status_cb(const struct ofono_error *error,
-					int status, int lac, int ci, int tech,
-					void *data)
+					int status, void *data)
 {
 	struct ofono_gprs *gprs = data;
 
 	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
-		ofono_gprs_status_notify(gprs, status, lac, ci, tech);
+		ofono_gprs_status_notify(gprs, status);
 
 	gprs->flags &= ~GPRS_FLAG_ATTACHING;
+
+	if (gprs->flags & GPRS_FLAG_RECHECK) {
+		gprs->flags &= ~GPRS_FLAG_RECHECK;
+		gprs_netreg_update(gprs);
+	}
 }
 
 static void gprs_attach_callback(const struct ofono_error *error, void *data)
 {
 	struct ofono_gprs *gprs = data;
 
-	if (error->type == OFONO_ERROR_TYPE_NO_ERROR) {
-		gprs->driver_attached = !gprs->driver_attached;
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
 		gprs_attached_update(gprs);
-	}
+	else
+		gprs->driver_attached = !gprs->driver_attached;
 
-	if (gprs->driver->registration_status) {
-		gprs->driver->registration_status(gprs, registration_status_cb,
-							gprs);
+	if (gprs->driver->attached_status) {
+		gprs->driver->attached_status(gprs, registration_status_cb,
+						gprs);
 		return;
 	}
 
 	gprs->flags &= ~GPRS_FLAG_ATTACHING;
+
+	if (gprs->flags & GPRS_FLAG_RECHECK) {
+		gprs->flags &= ~GPRS_FLAG_RECHECK;
+		gprs_netreg_update(gprs);
+	}
 }
 
 static void gprs_netreg_update(struct ofono_gprs *gprs)
@@ -1040,12 +944,15 @@ static void gprs_netreg_update(struct ofono_gprs *gprs)
 	if (gprs->driver_attached == attach)
 		return;
 
-	if (gprs->flags & GPRS_FLAG_ATTACHING)
+	if (gprs->flags & GPRS_FLAG_ATTACHING) {
+		gprs->flags |= GPRS_FLAG_RECHECK;
 		return;
+	}
 
 	gprs->flags |= GPRS_FLAG_ATTACHING;
 
 	gprs->driver->set_attached(gprs, attach, gprs_attach_callback, gprs);
+	gprs->driver_attached = attach;
 }
 
 static void netreg_status_changed(int status, int lac, int ci, int tech,
@@ -1073,7 +980,6 @@ static DBusMessage *gprs_get_properties(DBusConnection *conn,
 	DBusMessageIter dict;
 	char **objpath_list;
 	dbus_bool_t value;
-	const char *status = registration_status_to_string(gprs->status);
 
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
@@ -1104,28 +1010,6 @@ static DBusMessage *gprs_get_properties(DBusConnection *conn,
 	value = gprs->powered;
 	ofono_dbus_dict_append(&dict, "Powered", DBUS_TYPE_BOOLEAN, &value);
 
-	ofono_dbus_dict_append(&dict, "Status", DBUS_TYPE_STRING, &status);
-
-	if (gprs->location != -1) {
-		dbus_uint16_t location = gprs->location;
-		ofono_dbus_dict_append(&dict, "LocationAreaCode",
-					DBUS_TYPE_UINT16, &location);
-	}
-
-	if (gprs->cellid != -1) {
-		dbus_uint32_t cellid = gprs->cellid;
-		ofono_dbus_dict_append(&dict, "CellId",
-					DBUS_TYPE_UINT32, &cellid);
-	}
-
-	if (gprs->technology != -1) {
-		const char *technology =
-			registration_tech_to_string(gprs->technology);
-
-		ofono_dbus_dict_append(&dict, "Technology", DBUS_TYPE_STRING,
-					&technology);
-	}
-
 	dbus_message_iter_close_container(&iter, &dict);
 
 	return reply;
@@ -1200,8 +1084,9 @@ static DBusMessage *gprs_set_property(DBusConnection *conn,
 		}
 
 		gprs_netreg_update(gprs);
-	} else
+	} else {
 		return __ofono_error_invalid_args(msg);
+	}
 
 	path = __ofono_atom_get_path(gprs->atom);
 	ofono_dbus_signal_property_changed(conn, path,
@@ -1221,6 +1106,7 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 	const char *path;
 	enum gprs_context_type type;
 	char **objpath_list;
+	unsigned int id;
 
 	if (!dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &name,
 					DBUS_TYPE_STRING, &typestr,
@@ -1235,23 +1121,31 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 	if (type == GPRS_CONTEXT_TYPE_INVALID)
 		return __ofono_error_invalid_format(msg);
 
-	if (g_slist_length(gprs->contexts) >= MAX_CONTEXTS)
+	if (gprs->last_context_id)
+		id = idmap_alloc_next(gprs->pid_map, gprs->last_context_id);
+	else
+		id = idmap_alloc(gprs->pid_map);
+
+	if (id > idmap_get_max(gprs->pid_map))
 		return __ofono_error_not_supported(msg);
 
 	context = pri_context_create(gprs, name, type);
+	context->id = id;
 
 	if (!context) {
 		ofono_error("Unable to allocate context struct");
 		return __ofono_error_failed(msg);
 	}
 
-	ofono_debug("Registering new context");
+	DBG("Registering new context");
 
 	if (!context_dbus_register(context)) {
 		ofono_error("Unable to register primary context");
 		return __ofono_error_failed(msg);
 	}
 
+	gprs->last_context_id = id;
+
 	if (gprs->settings) {
 		g_key_file_set_string(gprs->settings, context->key,
 					"Name", context->name);
@@ -1295,7 +1189,7 @@ static void gprs_deactivate_for_remove(const struct ofono_error *error,
 	char **objpath_list;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Removing context failed with error: %s",
+		DBG("Removing context failed with error: %s",
 				telephony_error_to_str(error));
 
 		__ofono_dbus_pending_reply(&gprs->pending,
@@ -1361,7 +1255,7 @@ static DBusMessage *gprs_remove_context(DBusConnection *conn,
 		storage_sync(gprs->imsi, SETTINGS_STORE, gprs->settings);
 	}
 
-	ofono_debug("Unregistering context: %s\n", ctx->path);
+	DBG("Unregistering context: %s\n", ctx->path);
 	context_dbus_unregister(ctx);
 	gprs->contexts = g_slist_remove(gprs->contexts, ctx);
 
@@ -1425,26 +1319,13 @@ void ofono_gprs_detached_notify(struct ofono_gprs *gprs)
 	 */
 }
 
-void ofono_gprs_status_notify(struct ofono_gprs *gprs,
-				int status, int lac, int ci, int tech)
+void ofono_gprs_status_notify(struct ofono_gprs *gprs, int status)
 {
 	/* If we are not attached and haven't tried to attach, ignore */
-	if (gprs->driver_attached == FALSE &&
-			(gprs->flags & GPRS_FLAG_ATTACHING) == 0)
+	if (gprs->driver_attached == FALSE)
 		return;
 
-	if (gprs->status != status)
-		set_registration_status(gprs, status);
-
-	if (gprs->location != lac)
-		set_registration_location(gprs, lac);
-
-	if (gprs->cellid != ci)
-		set_registration_cellid(gprs, ci);
-
-	if (gprs->technology != tech)
-		set_registration_technology(gprs, tech);
-
+	gprs->status = status;
 	gprs_attached_update(gprs);
 }
 
@@ -1454,11 +1335,10 @@ void ofono_gprs_set_cid_range(struct ofono_gprs *gprs,
 	if (gprs == NULL)
 		return;
 
-	g_free(gprs->cid_bitmap);
-	gprs->cid_bitmap = g_new0(unsigned char, (max - min + 8) / 8);
+	if (gprs->cid_map)
+		idmap_free(gprs->cid_map);
 
-	gprs->cid_min = min;
-	gprs->cid_max = max;
+	gprs->cid_map = idmap_new_from_range(min, max);
 }
 
 static void gprs_context_unregister(struct ofono_atom *atom)
@@ -1643,6 +1523,11 @@ static void gprs_unregister(struct ofono_atom *atom)
 
 	g_slist_free(gprs->contexts);
 
+	if (gprs->cid_map) {
+		idmap_free(gprs->cid_map);
+		gprs->cid_map = NULL;
+	}
+
 	if (gprs->netreg_watch) {
 		if (gprs->status_watch) {
 			__ofono_netreg_remove_status_watch(gprs->netreg,
@@ -1669,6 +1554,11 @@ static void gprs_remove(struct ofono_atom *atom)
 	if (gprs == NULL)
 		return;
 
+	if (gprs->pid_map) {
+		idmap_free(gprs->pid_map);
+		gprs->pid_map = NULL;
+	}
+
 	if (gprs->context_driver) {
 		gprs->context_driver->gprs = NULL;
 		gprs->context_driver = NULL;
@@ -1713,10 +1603,7 @@ struct ofono_gprs *ofono_gprs_create(struct ofono_modem *modem,
 
 	gprs->status = NETWORK_REGISTRATION_STATUS_UNKNOWN;
 	gprs->netreg_status = NETWORK_REGISTRATION_STATUS_UNKNOWN;
-	gprs->technology = -1;
-	gprs->cellid = -1;
-	gprs->location = -1;
-	gprs->next_context_id = 1;
+	gprs->pid_map = idmap_new(MAX_CONTEXTS);
 
 	return gprs;
 }
@@ -1751,64 +1638,75 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 	gboolean ret = FALSE;
 	struct pri_context *context;
 	enum gprs_context_type type;
+	unsigned int id;
+
+	if (sscanf(group, "primarycontext%d", &id) != 1)
+		goto error;
+
+	if (id < 1 || id > MAX_CONTEXTS)
+		goto error;
 
 	if ((name = g_key_file_get_string(gprs->settings, group,
 					"Name", NULL)) == NULL)
-		goto err;
+		goto error;
 
 	if ((typestr = g_key_file_get_string(gprs->settings, group,
 					"Type", NULL)) == NULL)
-		goto err;
+		goto error;
 
 	type = gprs_context_string_to_type(typestr);
 	if (type == GPRS_CONTEXT_TYPE_INVALID)
-		goto err;
+		goto error;
 
 	username = g_key_file_get_string(gprs->settings, group,
 						"Username", NULL);
 	if (!username)
-		goto err;
+		goto error;
 
 	if (strlen(username) > OFONO_GPRS_MAX_USERNAME_LENGTH)
-		goto err;
+		goto error;
 
 	password = g_key_file_get_string(gprs->settings, group,
 						"Password", NULL);
 
 	if (!password)
-		goto err;
+		goto error;
 
 	if (strlen(password) > OFONO_GPRS_MAX_PASSWORD_LENGTH)
-		goto err;
+		goto error;
 
 	apn = g_key_file_get_string(gprs->settings, group,
 					"AccessPointName", NULL);
 
 	if (!apn)
-		goto err;
+		goto error;
 
 	if (strlen(apn) > OFONO_GPRS_MAX_APN_LENGTH)
-		goto err;
+		goto error;
 
 	/* Accept empty (just created) APNs, but don't allow other
 	 * invalid ones */
 	if (apn[0] != '\0' && is_valid_apn(apn) == FALSE)
-		goto err;
+		goto error;
 
 	if ((context = pri_context_create(gprs, name, type)) == NULL)
-		goto err;
+		goto error;
 
+	idmap_take(gprs->pid_map, id);
+	context->id = id;
 	strcpy(context->context.username, username);
 	strcpy(context->context.password, password);
 	strcpy(context->context.apn, apn);
 
 	if (context_dbus_register(context) == FALSE)
-		goto err;
+		goto error;
+
+	gprs->last_context_id = id;
 
 	gprs->contexts = g_slist_append(gprs->contexts, context);
 	ret = TRUE;
 
-err:
+error:
 	g_free(name);
 	g_free(typestr);
 	g_free(username);
