@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -39,30 +39,36 @@ static GSList *g_driver_list = NULL;
 static GSList *g_modem_list = NULL;
 
 static int next_modem_id = 0;
+static gboolean powering_down = FALSE;
+static int modems_remaining = 0;
 
 enum ofono_property_type {
 	OFONO_PROPERTY_TYPE_INVALID = 0,
 	OFONO_PROPERTY_TYPE_STRING,
 	OFONO_PROPERTY_TYPE_INTEGER,
+	OFONO_PROPERTY_TYPE_BOOLEAN,
 };
 
 struct ofono_modem {
-	char		*path;
-	GSList		*atoms;
-	GSList		*atom_watches;
-	int		next_atom_watch_id;
-	GSList         	*interface_list;
-	unsigned int	call_ids;
-	DBusMessage	*pending;
-	guint		interface_update;
-	ofono_bool_t	powered;
-	ofono_bool_t	powered_pending;
-	ofono_bool_t	powered_persistent;
-	guint		timeout;
-	GHashTable	*properties;
+	char			*path;
+	GSList			*atoms;
+	GSList			*pre_sim_atoms;
+	struct ofono_watchlist	*atom_watches;
+	GSList			*interface_list;
+	unsigned int		call_ids;
+	DBusMessage		*pending;
+	guint			interface_update;
+	ofono_bool_t		powered;
+	ofono_bool_t		powered_pending;
+	guint			timeout;
+	GHashTable		*properties;
+	struct ofono_sim	*sim;
+	unsigned int		sim_watch;
+	unsigned int		sim_ready_watch;
 	const struct ofono_modem_driver *driver;
-	void		*driver_data;
-	char		*driver_type;
+	void			*driver_data;
+	char			*driver_type;
+	char			*name;
 };
 
 struct ofono_devinfo {
@@ -84,11 +90,8 @@ struct ofono_atom {
 };
 
 struct ofono_atom_watch {
+	struct ofono_watchlist_item item;
 	enum ofono_atom_type type;
-	int id;
-	ofono_atom_watch_func notify;
-	ofono_destroy_func destroy;
-	void *notify_data;
 };
 
 struct ofono_property {
@@ -96,7 +99,7 @@ struct ofono_property {
 	void *value;
 };
 
-unsigned int __ofono_modem_alloc_callid(struct ofono_modem *modem)
+unsigned int __ofono_modem_callid_next(struct ofono_modem *modem)
 {
 	unsigned int i;
 
@@ -104,14 +107,18 @@ unsigned int __ofono_modem_alloc_callid(struct ofono_modem *modem)
 		if (modem->call_ids & (0x1 << i))
 			continue;
 
-		modem->call_ids |= (0x1 << i);
 		return i;
 	}
 
 	return 0;
 }
 
-void __ofono_modem_release_callid(struct ofono_modem *modem, int id)
+void __ofono_modem_callid_hold(struct ofono_modem *modem, int id)
+{
+	modem->call_ids |= (0x1 << id);
+}
+
+void __ofono_modem_callid_release(struct ofono_modem *modem, int id)
 {
 	modem->call_ids &= ~(0x1 << id);
 }
@@ -181,16 +188,19 @@ static void call_watches(struct ofono_atom *atom,
 				enum ofono_atom_watch_condition cond)
 {
 	struct ofono_modem *modem = atom->modem;
+	GSList *atom_watches = modem->atom_watches->items;
 	GSList *l;
 	struct ofono_atom_watch *watch;
+	ofono_atom_watch_func notify;
 
-	for (l = modem->atom_watches; l; l = l->next) {
+	for (l = atom_watches; l; l = l->next) {
 		watch = l->data;
 
 		if (watch->type != atom->type)
 			continue;
 
-		watch->notify(atom, cond, watch->notify_data);
+		notify = watch->item.notify;
+		notify(atom, cond, watch->item.notify_data);
 	}
 }
 
@@ -220,7 +230,7 @@ gboolean __ofono_atom_get_registered(struct ofono_atom *atom)
 	return atom->unregister ? TRUE : FALSE;
 }
 
-int __ofono_modem_add_atom_watch(struct ofono_modem *modem,
+unsigned int __ofono_modem_add_atom_watch(struct ofono_modem *modem,
 					enum ofono_atom_type type,
 					ofono_atom_watch_func notify,
 					void *data, ofono_destroy_func destroy)
@@ -233,68 +243,27 @@ int __ofono_modem_add_atom_watch(struct ofono_modem *modem,
 	watch = g_new0(struct ofono_atom_watch, 1);
 
 	watch->type = type;
-	watch->id = ++modem->next_atom_watch_id;
-	watch->notify = notify;
-	watch->destroy = destroy;
-	watch->notify_data = data;
+	watch->item.notify = notify;
+	watch->item.destroy = destroy;
+	watch->item.notify_data = data;
 
-	modem->atom_watches = g_slist_prepend(modem->atom_watches, watch);
-
-	return watch->id;
+	return __ofono_watchlist_add_item(modem->atom_watches,
+					(struct ofono_watchlist_item *)watch);
 }
 
-gboolean __ofono_modem_remove_atom_watch(struct ofono_modem *modem, int id)
+gboolean __ofono_modem_remove_atom_watch(struct ofono_modem *modem,
+						unsigned int id)
 {
-	struct ofono_atom_watch *watch;
-	GSList *p;
-	GSList *c;
-
-	p = NULL;
-	c = modem->atom_watches;
-
-	while (c) {
-		watch = c->data;
-
-		if (watch->id != id) {
-			p = c;
-			c = c->next;
-			continue;
-		}
-
-		if (p)
-			p->next = c->next;
-		else
-			modem->atom_watches = c->next;
-
-		if (watch->destroy)
-			watch->destroy(watch->notify_data);
-
-		g_free(watch);
-		g_slist_free_1(c);
-
-		return TRUE;
-	}
-
-	return FALSE;
+	return __ofono_watchlist_remove_item(modem->atom_watches, id);
 }
 
-static void remove_all_watches(struct ofono_modem *modem)
-{
-	struct ofono_atom_watch *watch;
-	GSList *l;
-
-	for (l = modem->atom_watches; l; l = l->next) {
-		watch = l->data;
-
-		if (watch->destroy)
-			watch->destroy(watch->notify_data);
-
-		g_free(watch);
-	}
-
-	g_slist_free(modem->atom_watches);
-	modem->atom_watches = NULL;
-}
+#define FIND_ATOM_IN_LIST(list)			\
+	for (l = list; l; l = l->next) {	\
+		atom = l->data;			\
+						\
+		if (atom->type == type)		\
+			return atom;		\
+	}					\
 
 struct ofono_atom *__ofono_modem_find_atom(struct ofono_modem *modem,
 						enum ofono_atom_type type)
@@ -305,16 +274,22 @@ struct ofono_atom *__ofono_modem_find_atom(struct ofono_modem *modem,
 	if (modem == NULL)
 		return NULL;
 
-	for (l = modem->atoms; l; l = l->next) {
-		atom = l->data;
-
-		if (atom->type == type)
-			return atom;
-	}
+	FIND_ATOM_IN_LIST(modem->atoms)
+	FIND_ATOM_IN_LIST(modem->pre_sim_atoms);
 
 	return NULL;
 }
 
+#define FOREACH_ATOM_IN_LIST(list)		\
+	for (l = list; l; l = l->next) {	\
+		atom = l->data;			\
+						\
+		if (atom->type != type)		\
+			continue;		\
+						\
+		callback(atom, data);		\
+	}					\
+
 void __ofono_modem_foreach_atom(struct ofono_modem *modem,
 				enum ofono_atom_type type,
 				ofono_atom_func callback, void *data)
@@ -325,14 +300,8 @@ void __ofono_modem_foreach_atom(struct ofono_modem *modem,
 	if (modem == NULL)
 		return;
 
-	for (l = modem->atoms; l; l = l->next) {
-		atom = l->data;
-
-		if (atom->type != type)
-			continue;
-
-		callback(atom, data);
-	}
+	FOREACH_ATOM_IN_LIST(modem->atoms)
+	FOREACH_ATOM_IN_LIST(modem->pre_sim_atoms)
 }
 
 void __ofono_atom_free(struct ofono_atom *atom)
@@ -340,6 +309,7 @@ void __ofono_atom_free(struct ofono_atom *atom)
 	struct ofono_modem *modem = atom->modem;
 
 	modem->atoms = g_slist_remove(modem->atoms, atom);
+	modem->pre_sim_atoms = g_slist_remove(modem->pre_sim_atoms, atom);
 
 	__ofono_atom_unregister(atom);
 
@@ -349,15 +319,12 @@ void __ofono_atom_free(struct ofono_atom *atom)
 	g_free(atom);
 }
 
-static void remove_all_atoms(struct ofono_modem *modem)
+static void remove_all_atoms(GSList **atoms)
 {
 	GSList *l;
 	struct ofono_atom *atom;
 
-	if (modem == NULL)
-		return;
-
-	for (l = modem->atoms; l; l = l->next) {
+	for (l = *atoms; l; l = l->next) {
 		atom = l->data;
 
 		__ofono_atom_unregister(atom);
@@ -368,8 +335,8 @@ static void remove_all_atoms(struct ofono_modem *modem)
 		g_free(atom);
 	}
 
-	g_slist_free(modem->atoms);
-	modem->atoms = NULL;
+	g_slist_free(*atoms);
+	*atoms = NULL;
 }
 
 static DBusMessage *modem_get_properties(DBusConnection *conn,
@@ -435,6 +402,10 @@ static DBusMessage *modem_get_properties(DBusConnection *conn,
 
 	g_free(interfaces);
 
+	if (modem->name)
+		ofono_dbus_dict_append(&dict, "Name", DBUS_TYPE_STRING,
+					&modem->name);
+
 	dbus_message_iter_close_container(&iter, &dict);
 
 	return reply;
@@ -445,14 +416,21 @@ static int set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 	const struct ofono_modem_driver *driver = modem->driver;
 	int err = -EINVAL;
 
-	if (driver == NULL)
-		return -EINVAL;
-
 	if (modem->powered_pending == powered)
 		return -EALREADY;
 
+	/* Remove the atoms even if the driver is no longer available */
+	if (powered == FALSE) {
+		remove_all_atoms(&modem->atoms);
+		remove_all_atoms(&modem->pre_sim_atoms);
+		modem->call_ids = 0;
+	}
+
 	modem->powered_pending = powered;
 
+	if (driver == NULL)
+		return -EINVAL;
+
 	if (powered == TRUE) {
 		if (driver->enable)
 			err = driver->enable(modem);
@@ -461,6 +439,11 @@ static int set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 			err = driver->disable(modem);
 	}
 
+	if (err == 0)
+		modem->powered = powered;
+	else if (err != -EINPROGRESS)
+		modem->powered_pending = modem->powered;
+
 	return err;
 }
 
@@ -472,6 +455,19 @@ static gboolean set_powered_timeout(gpointer user)
 
 	modem->timeout = 0;
 
+	if (modem->powered_pending == FALSE) {
+		DBusConnection *conn = ofono_dbus_get_connection();
+		dbus_bool_t powered = FALSE;
+
+		modem->powered = FALSE;
+		ofono_dbus_signal_property_changed(conn, modem->path,
+						OFONO_MODEM_INTERFACE,
+						"Powered", DBUS_TYPE_BOOLEAN,
+						&powered);
+	} else {
+		modem->powered_pending = modem->powered;
+	}
+
 	if (modem->pending != NULL) {
 		DBusMessage *reply;
 
@@ -501,6 +497,9 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT)
 		return __ofono_error_invalid_args(msg);
 
+	if (powering_down == TRUE)
+		return __ofono_error_failed(msg);
+
 	dbus_message_iter_recurse(&iter, &var);
 
 	if (g_str_equal(name, "Powered") == TRUE) {
@@ -529,11 +528,6 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 			return NULL;
 		}
 
-		ofono_debug("Foobar");
-
-		modem->powered = powered;
-		modem->powered_pending = powered;
-
 		g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
 
 		ofono_dbus_signal_property_changed(conn, modem->path,
@@ -542,12 +536,8 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 						&powered);
 
 		if (powered) {
-			if (modem->driver->populate)
-				modem->driver->populate(modem);
-
-			__ofono_history_probe_drivers(modem);
-		} else {
-			remove_all_atoms(modem);
+			if (modem->driver->pre_sim)
+				modem->driver->pre_sim(modem);
 		}
 
 		return NULL;
@@ -571,7 +561,6 @@ static GDBusSignalTable modem_signals[] = {
 void ofono_modem_set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
-	dbus_bool_t dbus_powered;
 
 	if (modem->timeout > 0) {
 		g_source_remove(modem->timeout);
@@ -591,27 +580,37 @@ void ofono_modem_set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 
 	modem->powered_pending = powered;
 
-	if (modem->powered == powered)
-		return;
+	if (modem->powered != powered) {
+		dbus_bool_t dbus_powered = powered;
+		modem->powered = powered;
 
-	modem->powered = powered;
+		if (modem->driver == NULL) {
+			ofono_error("Calling ofono_modem_set_powered on a"
+					"modem with no driver is not valid, "
+					"please fix the modem driver.");
+			return;
+		}
 
-	if (modem->driver == NULL)
-		return;
-
-	dbus_powered = powered;
-	ofono_dbus_signal_property_changed(conn, modem->path,
+		ofono_dbus_signal_property_changed(conn, modem->path,
 						OFONO_MODEM_INTERFACE,
 						"Powered", DBUS_TYPE_BOOLEAN,
 						&dbus_powered);
 
-	if (powered) {
-		if (modem->driver->populate)
-			modem->driver->populate(modem);
+		if (powered) {
+			if (modem->driver->pre_sim)
+				modem->driver->pre_sim(modem);
+		} else {
+			remove_all_atoms(&modem->atoms);
+			remove_all_atoms(&modem->pre_sim_atoms);
+			modem->call_ids = 0;
+		}
+	}
 
-		__ofono_history_probe_drivers(modem);
-	} else {
-		remove_all_atoms(modem);
+	if (powering_down && powered == FALSE) {
+		modems_remaining -= 1;
+
+		if (modems_remaining == 0)
+			__ofono_exit();
 	}
 }
 
@@ -785,7 +784,7 @@ static void query_manufacturer_cb(const struct ofono_error *error,
 
 	ofono_dbus_signal_property_changed(conn, path,
 						OFONO_MODEM_INTERFACE,
-						"Serial", DBUS_TYPE_STRING,
+						"Manufacturer", DBUS_TYPE_STRING,
 						&info->manufacturer);
 
 out:
@@ -896,7 +895,12 @@ void *ofono_devinfo_get_data(struct ofono_devinfo *info)
 	return info->driver_data;
 }
 
-/* Clients only need to free *modems */
+/* Clients only need to free *modems
+ *
+ * Note: this function will never return NULL. It will abort if it
+ * can't allocate memory for *modems; thus even an empty list will
+ * returns a non-NULL pointer.
+ */
 const char **__ofono_modem_get_list()
 {
 	GSList *l;
@@ -906,13 +910,13 @@ const char **__ofono_modem_get_list()
 
 	modems = g_new0(const char *, g_slist_length(g_modem_list) + 1);
 
-	for (l = g_modem_list, i = 0; l; l = l->next, i++) {
+	for (l = g_modem_list, i = 0; l; l = l->next) {
 		modem = l->data;
 
 		if (modem->driver == NULL)
 			continue;
 
-		modems[i] = modem->path;
+		modems[i++] = modem->path;
 	}
 
 	return modems;
@@ -953,6 +957,9 @@ static int set_modem_property(struct ofono_modem *modem, const char *name,
 	case OFONO_PROPERTY_TYPE_INTEGER:
 		property->value = g_memdup(value, sizeof(int));
 		break;
+	case OFONO_PROPERTY_TYPE_BOOLEAN:
+		property->value = g_memdup(value, sizeof(bool));
+		break;
 	default:
 		break;
 	}
@@ -985,6 +992,9 @@ static gboolean get_modem_property(struct ofono_modem *modem, const char *name,
 	case OFONO_PROPERTY_TYPE_INTEGER:
 		memcpy(value, property->value, sizeof(int));
 		return TRUE;
+	case OFONO_PROPERTY_TYPE_BOOLEAN:
+		memcpy(value, property->value, sizeof(bool));
+		return TRUE;
 	default:
 		return FALSE;
 	}
@@ -1004,6 +1014,13 @@ int ofono_modem_set_integer(struct ofono_modem *modem,
 					OFONO_PROPERTY_TYPE_INTEGER, &value);
 }
 
+int ofono_modem_set_boolean(struct ofono_modem *modem,
+				const char *key, bool value)
+{
+	return set_modem_property(modem, key,
+					OFONO_PROPERTY_TYPE_BOOLEAN, &value);
+}
+
 const char *ofono_modem_get_string(struct ofono_modem *modem, const char *key)
 {
 	const char *value;
@@ -1026,17 +1043,51 @@ int ofono_modem_get_integer(struct ofono_modem *modem, const char *key)
 	return value;
 }
 
-struct ofono_modem *ofono_modem_create(const char *type)
+bool ofono_modem_get_boolean(struct ofono_modem *modem, const char *key)
+{
+	bool value;
+
+	if (get_modem_property(modem, key,
+			OFONO_PROPERTY_TYPE_BOOLEAN, &value) == FALSE)
+		return FALSE;
+
+	return value;
+}
+
+void ofono_modem_set_name(struct ofono_modem *modem, const char *name)
+{
+	if (modem->name)
+		g_free(modem->name);
+
+	modem->name = g_strdup(name);
+
+	if (modem->driver) {
+		DBusConnection *conn = ofono_dbus_get_connection();
+
+		ofono_dbus_signal_property_changed(conn, modem->path,
+						OFONO_MODEM_INTERFACE,
+						"Name", DBUS_TYPE_STRING,
+						&modem->name);
+	}
+}
+
+struct ofono_modem *ofono_modem_create(const char *name, const char *type)
 {
 	struct ofono_modem *modem;
 	char path[128];
 
-	DBG("%s", type);
+	DBG("name: %s, type: %s", name, type);
 
 	if (strlen(type) > 16)
 		return NULL;
 
-	snprintf(path, sizeof(path), "/%s%d", type, next_modem_id);
+	if (name && strlen(name) > 64)
+		return NULL;
+
+	if (name == NULL)
+		snprintf(path, sizeof(path), "/%s%d", type, next_modem_id);
+	else
+		snprintf(path, sizeof(path), "/%s", name);
 
 	if (__ofono_dbus_valid_object_path(path) == FALSE)
 		return NULL;
@@ -1053,7 +1104,8 @@ struct ofono_modem *ofono_modem_create(const char *type)
 
 	g_modem_list = g_slist_prepend(g_modem_list, modem);
 
-	next_modem_id += 1;
+	if (name == NULL)
+		next_modem_id += 1;
 
 	return modem;
 }
@@ -1074,6 +1126,47 @@ static void emit_modems()
 	g_free(modems);
 }
 
+static void modem_sim_ready(void *user, enum ofono_sim_state new_state)
+{
+	struct ofono_modem *modem = user;
+
+	switch (new_state) {
+	case OFONO_SIM_STATE_NOT_PRESENT:
+		if (modem->pre_sim_atoms != NULL)
+			remove_all_atoms(&modem->atoms);
+		break;
+	case OFONO_SIM_STATE_INSERTED:
+		break;
+	case OFONO_SIM_STATE_READY:
+		if (modem->pre_sim_atoms == NULL) {
+			modem->pre_sim_atoms = modem->atoms;
+			modem->atoms = NULL;
+		}
+
+		if (modem->driver->post_sim)
+			modem->driver->post_sim(modem);
+
+		__ofono_history_probe_drivers(modem);
+		__ofono_nettime_probe_drivers(modem);
+	}
+}
+
+static void sim_watch(struct ofono_atom *atom,
+			enum ofono_atom_watch_condition cond, void *data)
+{
+	struct ofono_modem *modem = data;
+
+	if (cond == OFONO_ATOM_WATCH_CONDITION_UNREGISTERED) {
+		modem->sim_ready_watch = 0;
+		return;
+	}
+
+	modem->sim = __ofono_atom_get_data(atom);
+	modem->sim_ready_watch = ofono_sim_add_state_watch(modem->sim,
+							modem_sim_ready,
+							modem, NULL);
+}
+
 int ofono_modem_register(struct ofono_modem *modem)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
@@ -1082,6 +1175,9 @@ int ofono_modem_register(struct ofono_modem *modem)
 	if (modem == NULL)
 		return -EINVAL;
 
+	if (powering_down == TRUE)
+		return -EBUSY;
+
 	if (modem->driver != NULL)
 		return -EALREADY;
 
@@ -1117,16 +1213,13 @@ int ofono_modem_register(struct ofono_modem *modem)
 	g_free(modem->driver_type);
 	modem->driver_type = NULL;
 
+	modem->atom_watches = __ofono_watchlist_new(g_free);
+
 	emit_modems();
 
-	/* TODO: Read powered property from store */
-	if (modem->powered_persistent)
-		set_powered(modem, TRUE);
-
-	if (modem->powered == TRUE && modem->driver->populate) {
-		modem->driver->populate(modem);
-		__ofono_history_probe_drivers(modem);
-	}
+	modem->sim_watch = __ofono_modem_add_atom_watch(modem,
+					OFONO_ATOM_TYPE_SIM,
+					sim_watch, modem, NULL);
 
 	return 0;
 }
@@ -1135,11 +1228,14 @@ static void modem_unregister(struct ofono_modem *modem)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 
-	if (modem->driver == NULL)
-		return;
+	if (modem->powered == TRUE)
+		set_powered(modem, FALSE);
 
-	remove_all_atoms(modem);
-	remove_all_watches(modem);
+	__ofono_watchlist_free(modem->atom_watches);
+	modem->atom_watches = NULL;
+
+	modem->sim_watch = 0;
+	modem->sim_ready_watch = 0;
 
 	g_slist_foreach(modem->interface_list, (GFunc)g_free, NULL);
 	g_slist_free(modem->interface_list);
@@ -1162,10 +1258,7 @@ static void modem_unregister(struct ofono_modem *modem)
 
 	g_dbus_unregister_interface(conn, modem->path, OFONO_MODEM_INTERFACE);
 
-	if (modem->powered == TRUE)
-		set_powered(modem, FALSE);
-
-	if (modem->driver->remove)
+	if (modem->driver && modem->driver->remove)
 		modem->driver->remove(modem);
 
 	g_hash_table_destroy(modem->properties);
@@ -1209,7 +1302,43 @@ int ofono_modem_driver_register(const struct ofono_modem_driver *d)
 
 void ofono_modem_driver_unregister(const struct ofono_modem_driver *d)
 {
+	GSList *l;
+	struct ofono_modem *modem;
+
 	DBG("driver: %p, name: %s", d, d->name);
 
 	g_driver_list = g_slist_remove(g_driver_list, (void *)d);
+
+	for (l = g_modem_list; l; l = l->next) {
+		modem = l->data;
+
+		if (modem->driver != d)
+			continue;
+
+		modem_unregister(modem);
+	}
+}
+
+void __ofono_modem_shutdown()
+{
+	struct ofono_modem *modem;
+	GSList *l;
+
+	powering_down = TRUE;
+
+	for (l = g_modem_list; l; l = l->next) {
+		modem = l->data;
+
+		if (modem->driver == NULL)
+			continue;
+
+		if (modem->powered == FALSE && modem->powered_pending == FALSE)
+			continue;
+
+		if (set_powered(modem, FALSE) == -EINPROGRESS)
+			modems_remaining += 1;
+	}
+
+	if (modems_remaining == 0)
+		__ofono_exit();
 }
